local Tinkr, _Bastion, DBM = ...

---@class Bastion
local Bastion = _Bastion

---@alias BigWigsBars table<string, { spellId: number, duration: number, startTime: number, pauseTime: number | nil }>

---@class MythicPlusUtils
local MythicPlusUtils = {
    ---@type InterruptList
    interruptList = {},
    ---@type BigWigsBars
    bars = {}
}

MythicPlusUtils.__index = MythicPlusUtils

local function BWEventCallBack(event, ...)
    if event == "BigWigs_StartBar" then
        local addon, spellId, text, duration, icon, isCD = ...
        local now = GetTime()

        MythicPlusUtils.bars[text] = MythicPlusUtils.bars[text] or {}

        MythicPlusUtils.bars[text].duration = duration
        MythicPlusUtils.bars[text].startTime = now
        MythicPlusUtils.bars[text].spellId = spellId
    elseif event == "BigWigs_StopBar" then
        local addon, text = ...
        if MythicPlusUtils.bars[text] then
            MythicPlusUtils.bars[text] = nil
        end
    elseif event == "BigWigs_PauseBar" then
        local addon, text = ...
        if MythicPlusUtils.bars[text] then
            MythicPlusUtils.bars[text].pauseTime = GetTime()
        end
    elseif event == "BigWigs_ResumeBar" then
        local addon, text = ...
        if MythicPlusUtils.bars[text] and MythicPlusUtils.bars[text].pauseTime then
            local pauseTime = MythicPlusUtils.bars[text].pauseTime
            local startTime = MythicPlusUtils.bars[text].startTime
            local duration = MythicPlusUtils.bars[text].duration

            local newDuration = duration - (pauseTime - startTime)

            MythicPlusUtils.bars[text].duration = newDuration
            MythicPlusUtils.bars[text].startTime = GetTime()
            MythicPlusUtils.bars[text].pauseTime = nil
        end
    elseif event == "BigWigs_StopBars" or event == "BigWigs_OnBossDisable" then
        MythicPlusUtils.bars = {}
    end
end

---@type InterruptList
local _interruptList = Tinkr:require("scripts/bastion/MythicPlusUtils/lists/interrupt-list", Bastion)

---@return MythicPlusUtils
function MythicPlusUtils:New()
    local self = setmetatable({}, MythicPlusUtils)

    self.interruptList = _interruptList

    if BigWigsLoader then
        BigWigsLoader.RegisterMessage({}, 'BigWigs_StartBar', BWEventCallBack)
        BigWigsLoader.RegisterMessage({}, 'BigWigs_StopBar', BWEventCallBack)
        BigWigsLoader.RegisterMessage({}, 'BigWigs_PauseBar', BWEventCallBack)
        BigWigsLoader.RegisterMessage({}, 'BigWigs_ResumeBar', BWEventCallBack)
        BigWigsLoader.RegisterMessage({}, 'BigWigs_StopBars', BWEventCallBack)
        BigWigsLoader.RegisterMessage({}, 'BigWigs_OnBossDisable', BWEventCallBack)
    end

    return self
end

---@param unit Unit
---@param percent number
---@return { kickable: boolean, paraliseable: boolean, sweepable: boolean } | nil
function MythicPlusUtils:CastingCriticalStop(unit, percent)
    local castingSpell = unit:GetCastingOrChannelingSpell()

    local npcSpells = self.interruptList[unit:GetID()]

    if npcSpells and castingSpell then
        local spellID = castingSpell:GetID()

        local spellTraits = npcSpells[spellID]

        if not spellTraits then
            return nil
        end

        local castPercent = unit:GetChannelOrCastPercentComplete()

        if castPercent >= percent then
            return spellTraits
        end
    end

    return nil
end

---@param unit Unit
---@param timeUntilCast? number
---@return boolean
function MythicPlusUtils:CastingTankBuster(unit, timeUntilCast)

    return false
end

return MythicPlusUtils
